@@ -196,33 +196,43 @@ extern "C" {
      * *******************************************************************/
     SEXP betaSamples_r;
     PROTECT(betaSamples_r = allocMatrix(REALSXP, pOcc, nPost)); nProtect++;
+    zeros(REAL(betaSamples_r), pOcc * nPost);
     SEXP alphaSamples_r; 
     PROTECT(alphaSamples_r = allocMatrix(REALSXP, pDet, nPost)); nProtect++;
+    zeros(REAL(alphaSamples_r), pDet * nPost);
     SEXP zSamples_r; 
     PROTECT(zSamples_r = allocMatrix(REALSXP, JnYears, nPost)); nProtect++; 
+    zeros(REAL(zSamples_r), JnYears * nPost);
     SEXP etaSamples_r; 
     if (ar1) {
       PROTECT(etaSamples_r = allocMatrix(REALSXP, nYearsMax, nPost)); nProtect++; 
+      zeros(REAL(etaSamples_r), nYearsMax * nPost);
     }
     SEXP psiSamples_r; 
     PROTECT(psiSamples_r = allocMatrix(REALSXP, JnYears, nPost)); nProtect++; 
+    zeros(REAL(psiSamples_r), JnYears * nPost);
     // Detection random effects
     SEXP sigmaSqPSamples_r; 
     SEXP alphaStarSamples_r; 
     if (pDetRE > 0) {
       PROTECT(sigmaSqPSamples_r = allocMatrix(REALSXP, pDetRE, nPost)); nProtect++;
+      zeros(REAL(sigmaSqPSamples_r), pDetRE * nPost);
       PROTECT(alphaStarSamples_r = allocMatrix(REALSXP, nDetRE, nPost)); nProtect++;
+      zeros(REAL(alphaStarSamples_r), nDetRE * nPost);
     }
     // Occurrence random effects
     SEXP sigmaSqPsiSamples_r; 
     SEXP betaStarSamples_r; 
     if (pOccRE > 0) {
       PROTECT(sigmaSqPsiSamples_r = allocMatrix(REALSXP, pOccRE, nPost)); nProtect++;
+      zeros(REAL(sigmaSqPsiSamples_r), pOccRE * nPost);
       PROTECT(betaStarSamples_r = allocMatrix(REALSXP, nOccRE, nPost)); nProtect++;
+      zeros(REAL(betaStarSamples_r), nOccRE * nPost);
     }
     // Likelihood samples for WAIC. 
     SEXP likeSamples_r;
     PROTECT(likeSamples_r = allocMatrix(REALSXP, JnYears, nPost)); nProtect++;
+    zeros(REAL(likeSamples_r), J * nPost);
     
     /**********************************************************************
      * Other initial starting stuff
@@ -339,6 +349,7 @@ extern "C" {
     SEXP thetaSamples_r; 
     if (ar1) {
       PROTECT(thetaSamples_r = allocMatrix(REALSXP, nTheta, nPost)); nProtect++; 
+      zeros(REAL(thetaSamples_r), nTheta * nPost);
     }
     // Initiate values
     theta[sigmaSqTIndx] = REAL(ar1Vals_r)[5];
